@@ -2,12 +2,12 @@ from __future__ import annotations as _annotations
 
 import sys
 from datetime import date, datetime, time, timedelta
-from typing import Any, Callable, Dict, List, Optional, Type, Union, overload
+from typing import Any, Callable, Dict, List, Optional, Set, Type, Union
 
 if sys.version_info < (3, 11):
-    from typing_extensions import NotRequired, Required
+    from typing_extensions import Protocol, Required
 else:
-    from typing import NotRequired, Required
+    from typing import Protocol, Required
 
 if sys.version_info < (3, 9):
     from typing_extensions import Literal, TypedDict
@@ -20,6 +20,7 @@ def dict_not_none(**kwargs: Any) -> Any:
 
 
 class CoreConfig(TypedDict, total=False):
+    title: str
     strict: bool
     # higher priority configs take precedence of over lower, if priority matches the two configs are merged, default 0
     config_choose_priority: int
@@ -43,31 +44,35 @@ class CoreConfig(TypedDict, total=False):
     allow_inf_nan: bool  # default: True
 
 
-class AnySchema(TypedDict):
-    type: Literal['any']
+class AnySchema(TypedDict, total=False):
+    type: Required[Literal['any']]
+    ref: str
+    extra: Any
 
 
-def any_schema() -> AnySchema:
-    return {'type': 'any'}
+def any_schema(*, ref: str | None = None, extra: Any = None) -> AnySchema:
+    return dict_not_none(type='any', ref=ref, extra=extra)
 
 
-class NoneSchema(TypedDict):
-    type: Literal['none']
-    ref: NotRequired[str]
+class NoneSchema(TypedDict, total=False):
+    type: Required[Literal['none']]
+    ref: str
+    extra: Any
 
 
-def none_schema(*, ref: str | None = None) -> NoneSchema:
-    return dict_not_none(type='none', ref=ref)
+def none_schema(*, ref: str | None = None, extra: Any = None) -> NoneSchema:
+    return dict_not_none(type='none', ref=ref, extra=extra)
 
 
 class BoolSchema(TypedDict, total=False):
     type: Required[Literal['bool']]
     strict: bool
     ref: str
+    extra: Any
 
 
-def bool_schema(strict: bool | None = None, ref: str | None = None) -> BoolSchema:
-    return dict_not_none(type='bool', strict=strict, ref=ref)
+def bool_schema(strict: bool | None = None, ref: str | None = None, extra: Any = None) -> BoolSchema:
+    return dict_not_none(type='bool', strict=strict, ref=ref, extra=extra)
 
 
 class IntSchema(TypedDict, total=False):
@@ -79,6 +84,7 @@ class IntSchema(TypedDict, total=False):
     gt: int
     strict: bool
     ref: str
+    extra: Any
 
 
 def int_schema(
@@ -90,8 +96,11 @@ def int_schema(
     gt: int | None = None,
     strict: bool | None = None,
     ref: str | None = None,
+    extra: Any = None,
 ) -> IntSchema:
-    return dict_not_none(type='int', multiple_of=multiple_of, le=le, ge=ge, lt=lt, gt=gt, strict=strict, ref=ref)
+    return dict_not_none(
+        type='int', multiple_of=multiple_of, le=le, ge=ge, lt=lt, gt=gt, strict=strict, ref=ref, extra=extra
+    )
 
 
 class FloatSchema(TypedDict, total=False):
@@ -104,6 +113,7 @@ class FloatSchema(TypedDict, total=False):
     gt: float
     strict: bool
     ref: str
+    extra: Any
 
 
 def float_schema(
@@ -116,6 +126,7 @@ def float_schema(
     gt: float | None = None,
     strict: bool | None = None,
     ref: str | None = None,
+    extra: Any = None,
 ) -> FloatSchema:
     return dict_not_none(
         type='float',
@@ -127,6 +138,7 @@ def float_schema(
         gt=gt,
         strict=strict,
         ref=ref,
+        extra=extra,
     )
 
 
@@ -140,6 +152,7 @@ class StringSchema(TypedDict, total=False):
     to_upper: bool
     strict: bool
     ref: str
+    extra: Any
 
 
 def string_schema(
@@ -152,6 +165,7 @@ def string_schema(
     to_upper: bool | None = None,
     strict: bool | None = None,
     ref: str | None = None,
+    extra: Any = None,
 ) -> StringSchema:
     return dict_not_none(
         type='str',
@@ -163,6 +177,7 @@ def string_schema(
         to_upper=to_upper,
         strict=strict,
         ref=ref,
+        extra=extra,
     )
 
 
@@ -172,12 +187,20 @@ class BytesSchema(TypedDict, total=False):
     min_length: int
     strict: bool
     ref: str
+    extra: Any
 
 
 def bytes_schema(
-    *, max_length: int | None = None, min_length: int | None = None, strict: bool | None = None, ref: str | None = None
+    *,
+    max_length: int | None = None,
+    min_length: int | None = None,
+    strict: bool | None = None,
+    ref: str | None = None,
+    extra: Any = None,
 ) -> BytesSchema:
-    return dict_not_none(type='bytes', max_length=max_length, min_length=min_length, strict=strict, ref=ref)
+    return dict_not_none(
+        type='bytes', max_length=max_length, min_length=min_length, strict=strict, ref=ref, extra=extra
+    )
 
 
 class DateSchema(TypedDict, total=False):
@@ -188,6 +211,7 @@ class DateSchema(TypedDict, total=False):
     lt: date
     gt: date
     ref: str
+    extra: Any
 
 
 def date_schema(
@@ -198,8 +222,9 @@ def date_schema(
     lt: date | None = None,
     gt: date | None = None,
     ref: str | None = None,
+    extra: Any = None,
 ) -> DateSchema:
-    return dict_not_none(type='date', strict=strict, le=le, ge=ge, lt=lt, gt=gt, ref=ref)
+    return dict_not_none(type='date', strict=strict, le=le, ge=ge, lt=lt, gt=gt, ref=ref, extra=extra)
 
 
 class TimeSchema(TypedDict, total=False):
@@ -210,6 +235,7 @@ class TimeSchema(TypedDict, total=False):
     lt: time
     gt: time
     ref: str
+    extra: Any
 
 
 def time_schema(
@@ -220,8 +246,9 @@ def time_schema(
     lt: time | None = None,
     gt: time | None = None,
     ref: str | None = None,
+    extra: Any = None,
 ) -> TimeSchema:
-    return dict_not_none(type='time', strict=strict, le=le, ge=ge, lt=lt, gt=gt, ref=ref)
+    return dict_not_none(type='time', strict=strict, le=le, ge=ge, lt=lt, gt=gt, ref=ref, extra=extra)
 
 
 class DatetimeSchema(TypedDict, total=False):
@@ -232,6 +259,7 @@ class DatetimeSchema(TypedDict, total=False):
     lt: datetime
     gt: datetime
     ref: str
+    extra: Any
 
 
 def datetime_schema(
@@ -242,8 +270,9 @@ def datetime_schema(
     lt: datetime | None = None,
     gt: datetime | None = None,
     ref: str | None = None,
+    extra: Any = None,
 ) -> DatetimeSchema:
-    return dict_not_none(type='datetime', strict=strict, le=le, ge=ge, lt=lt, gt=gt, ref=ref)
+    return dict_not_none(type='datetime', strict=strict, le=le, ge=ge, lt=lt, gt=gt, ref=ref, extra=extra)
 
 
 class TimedeltaSchema(TypedDict, total=False):
@@ -254,6 +283,7 @@ class TimedeltaSchema(TypedDict, total=False):
     lt: timedelta
     gt: timedelta
     ref: str
+    extra: Any
 
 
 def timedelta_schema(
@@ -264,44 +294,67 @@ def timedelta_schema(
     lt: timedelta | None = None,
     gt: timedelta | None = None,
     ref: str | None = None,
+    extra: Any = None,
 ) -> TimedeltaSchema:
-    return dict_not_none(type='timedelta', strict=strict, le=le, ge=ge, lt=lt, gt=gt, ref=ref)
+    return dict_not_none(type='timedelta', strict=strict, le=le, ge=ge, lt=lt, gt=gt, ref=ref, extra=extra)
 
 
-class LiteralSchema(TypedDict):
-    type: Literal['literal']
-    expected: List[Any]
-    ref: NotRequired[str]
+class LiteralSchema(TypedDict, total=False):
+    type: Required[Literal['literal']]
+    expected: Required[List[Any]]
+    ref: str
+    extra: Any
 
 
-def literal_schema(*expected: Any, ref: str | None = None) -> LiteralSchema:
-    return dict_not_none(type='literal', expected=expected, ref=ref)
+def literal_schema(*expected: Any, ref: str | None = None, extra: Any = None) -> LiteralSchema:
+    return dict_not_none(type='literal', expected=expected, ref=ref, extra=extra)
 
 
-class IsInstanceSchema(TypedDict):
-    type: Literal['is-instance']
-    cls: Type[Any]
+# must match input/parse_json.rs::JsonType::try_from
+JsonType = Literal['null', 'bool', 'int', 'float', 'str', 'list', 'dict']
 
 
-def is_instance_schema(cls: Type[Any]) -> IsInstanceSchema:
-    return dict_not_none(type='is-instance', cls=cls)
+class IsInstanceSchema(TypedDict, total=False):
+    type: Required[Literal['is-instance']]
+    cls: Required[Type[Any]]
+    json_types: Set[JsonType]
+    json_function: Callable[[Any], Any]
+    ref: str
+    extra: Any
 
 
-class CallableSchema(TypedDict):
-    type: Literal['callable']
+def is_instance_schema(
+    cls: Type[Any],
+    *,
+    json_types: Set[JsonType] | None = None,
+    json_function: Callable[[Any], Any] | None = None,
+    ref: str | None = None,
+    extra: Any = None,
+) -> IsInstanceSchema:
+    return dict_not_none(
+        type='is-instance', cls=cls, json_types=json_types, json_function=json_function, ref=ref, extra=extra
+    )
 
 
-def callable_schema() -> CallableSchema:
-    return dict_not_none(type='callable')
+class CallableSchema(TypedDict, total=False):
+    type: Required[Literal['callable']]
+    ref: str
+    extra: Any
+
+
+def callable_schema(*, ref: str | None = None, extra: Any = None) -> CallableSchema:
+    return dict_not_none(type='callable', ref=ref, extra=extra)
 
 
 class ListSchema(TypedDict, total=False):
     type: Required[Literal['list']]
-    items_schema: CoreSchema  # default: AnySchema
+    items_schema: CoreSchema
     min_length: int
     max_length: int
     strict: bool
+    allow_any_iter: bool
     ref: str
+    extra: Any
 
 
 def list_schema(
@@ -310,10 +363,19 @@ def list_schema(
     min_length: int | None = None,
     max_length: int | None = None,
     strict: bool | None = None,
+    allow_any_iter: bool | None = None,
     ref: str | None = None,
+    extra: Any = None,
 ) -> ListSchema:
     return dict_not_none(
-        type='list', items_schema=items_schema, min_length=min_length, max_length=max_length, strict=strict, ref=ref
+        type='list',
+        items_schema=items_schema,
+        min_length=min_length,
+        max_length=max_length,
+        strict=strict,
+        allow_any_iter=allow_any_iter,
+        ref=ref,
+        extra=extra,
     )
 
 
@@ -324,6 +386,7 @@ class TuplePositionalSchema(TypedDict, total=False):
     extra_schema: CoreSchema
     strict: bool
     ref: str
+    extra: Any
 
 
 def tuple_positional_schema(
@@ -331,9 +394,16 @@ def tuple_positional_schema(
     extra_schema: CoreSchema | None = None,
     strict: bool | None = None,
     ref: str | None = None,
+    extra: Any = None,
 ) -> TuplePositionalSchema:
     return dict_not_none(
-        type='tuple', mode='positional', items_schema=items_schema, extra_schema=extra_schema, strict=strict, ref=ref
+        type='tuple',
+        mode='positional',
+        items_schema=items_schema,
+        extra_schema=extra_schema,
+        strict=strict,
+        ref=ref,
+        extra=extra,
     )
 
 
@@ -345,6 +415,7 @@ class TupleVariableSchema(TypedDict, total=False):
     max_length: int
     strict: bool
     ref: str
+    extra: Any
 
 
 def tuple_variable_schema(
@@ -354,6 +425,7 @@ def tuple_variable_schema(
     max_length: int | None = None,
     strict: bool | None = None,
     ref: str | None = None,
+    extra: Any = None,
 ) -> TupleVariableSchema:
     return dict_not_none(
         type='tuple',
@@ -363,16 +435,19 @@ def tuple_variable_schema(
         max_length=max_length,
         strict=strict,
         ref=ref,
+        extra=extra,
     )
 
 
 class SetSchema(TypedDict, total=False):
     type: Required[Literal['set']]
-    items_schema: CoreSchema  # default: AnySchema
+    items_schema: CoreSchema
     min_length: int
     max_length: int
+    generator_max_length: int
     strict: bool
     ref: str
+    extra: Any
 
 
 def set_schema(
@@ -380,41 +455,70 @@ def set_schema(
     *,
     min_length: int | None = None,
     max_length: int | None = None,
+    generator_max_length: int | None = None,
     strict: bool | None = None,
     ref: str | None = None,
+    extra: Any = None,
 ) -> SetSchema:
     return dict_not_none(
-        type='set', items_schema=items_schema, min_length=min_length, max_length=max_length, strict=strict, ref=ref
+        type='set',
+        items_schema=items_schema,
+        min_length=min_length,
+        max_length=max_length,
+        generator_max_length=generator_max_length,
+        strict=strict,
+        ref=ref,
+        extra=extra,
     )
 
 
 class FrozenSetSchema(TypedDict, total=False):
     type: Required[Literal['frozenset']]
-    items_schema: CoreSchema  # default: AnySchema
+    items_schema: CoreSchema
     min_length: int
     max_length: int
+    generator_max_length: int
     strict: bool
     ref: str
+    extra: Any
 
 
-def frozen_set_schema(
+def frozenset_schema(
     items_schema: CoreSchema | None = None,
     *,
     min_length: int | None = None,
     max_length: int | None = None,
+    generator_max_length: int | None = None,
     strict: bool | None = None,
     ref: str | None = None,
+    extra: Any = None,
 ) -> FrozenSetSchema:
     return dict_not_none(
         type='frozenset',
         items_schema=items_schema,
         min_length=min_length,
         max_length=max_length,
+        generator_max_length=generator_max_length,
         strict=strict,
         ref=ref,
+        extra=extra,
     )
 
 
+class GeneratorSchema(TypedDict, total=False):
+    type: Required[Literal['generator']]
+    items_schema: CoreSchema
+    max_length: int
+    ref: str
+    extra: Any
+
+
+def generator_schema(
+    items_schema: CoreSchema | None = None, *, max_length: int | None = None, ref: str | None = None, extra: Any = None
+) -> GeneratorSchema:
+    return dict_not_none(type='generator', items_schema=items_schema, max_length=max_length, ref=ref, extra=extra)
+
+
 class DictSchema(TypedDict, total=False):
     type: Required[Literal['dict']]
     keys_schema: CoreSchema  # default: AnySchema
@@ -423,6 +527,7 @@ class DictSchema(TypedDict, total=False):
     max_length: int
     strict: bool
     ref: str
+    extra: Any
 
 
 def dict_schema(
@@ -433,6 +538,7 @@ def dict_schema(
     max_length: int | None = None,
     strict: bool | None = None,
     ref: str | None = None,
+    extra: Any = None,
 ) -> DictSchema:
     return dict_not_none(
         type='dict',
@@ -442,58 +548,84 @@ def dict_schema(
         max_length=max_length,
         strict=strict,
         ref=ref,
+        extra=extra,
     )
 
 
-class FunctionSchema(TypedDict):
-    type: Literal['function']
-    mode: Literal['before', 'after', 'wrap']
-    function: Callable[..., Any]
-    schema: CoreSchema
-    # validator_instance is used by pydantic for progressively preparing the function, ignored by pydantic-core
-    validator_instance: NotRequired[Any]
-    ref: NotRequired[str]
+class ValidatorFunction(Protocol):
+    def __call__(
+        self, __input_value: Any, *, data: Any, config: CoreConfig | None, context: Any, **future_kwargs: Any
+    ) -> Any:  # pragma: no cover
+        ...
+
+
+class FunctionSchema(TypedDict, total=False):
+    type: Required[Literal['function']]
+    mode: Required[Literal['before', 'after']]
+    function: Required[ValidatorFunction]
+    schema: Required[CoreSchema]
+    ref: str
+    extra: Any
 
 
 def function_before_schema(
-    function: Callable[..., Any], schema: CoreSchema, *, validator_instance: Any | None = None, ref: str | None = None
+    function: ValidatorFunction, schema: CoreSchema, *, ref: str | None = None, extra: Any = None
 ) -> FunctionSchema:
-    return dict_not_none(
-        type='function', mode='before', function=function, schema=schema, validator_instance=validator_instance, ref=ref
-    )
+    return dict_not_none(type='function', mode='before', function=function, schema=schema, ref=ref, extra=extra)
 
 
 def function_after_schema(
-    function: Callable[..., Any], schema: CoreSchema, *, validator_instance: Any | None = None, ref: str | None = None
+    schema: CoreSchema, function: ValidatorFunction, *, ref: str | None = None, extra: Any = None
 ) -> FunctionSchema:
-    return dict_not_none(
-        type='function', mode='after', function=function, schema=schema, validator_instance=validator_instance, ref=ref
-    )
+    return dict_not_none(type='function', mode='after', function=function, schema=schema, ref=ref, extra=extra)
+
+
+class CallableValidator(Protocol):
+    def __call__(self, input_value: Any, outer_location: str | int | None = None) -> Any:  # pragma: no cover
+        ...
+
+
+class WrapValidatorFunction(Protocol):
+    def __call__(
+        self,
+        __input_value: Any,
+        *,
+        validator: CallableValidator,
+        data: Any,
+        config: CoreConfig | None,
+        context: Any,
+        **future_kwargs: Any,
+    ) -> Any:  # pragma: no cover
+        ...
+
+
+class FunctionWrapSchema(TypedDict, total=False):
+    type: Required[Literal['function']]
+    mode: Required[Literal['wrap']]
+    function: Required[WrapValidatorFunction]
+    schema: Required[CoreSchema]
+    ref: str
+    extra: Any
 
 
 def function_wrap_schema(
-    function: Callable[..., Any], schema: CoreSchema, *, validator_instance: Any | None = None, ref: str | None = None
-) -> FunctionSchema:
-    return dict_not_none(
-        type='function', mode='wrap', function=function, schema=schema, validator_instance=validator_instance, ref=ref
-    )
+    function: WrapValidatorFunction, schema: CoreSchema, *, ref: str | None = None, extra: Any = None
+) -> FunctionWrapSchema:
+    return dict_not_none(type='function', mode='wrap', function=function, schema=schema, ref=ref, extra=extra)
 
 
-class FunctionPlainSchema(TypedDict):
-    type: Literal['function']
-    mode: Literal['plain']
-    function: Callable[..., Any]
-    # validator_instance is used by pydantic for progressively preparing the function, ignored by pydantic-core
-    validator_instance: NotRequired[Any]
-    ref: NotRequired[str]
+class FunctionPlainSchema(TypedDict, total=False):
+    type: Required[Literal['function']]
+    mode: Required[Literal['plain']]
+    function: Required[ValidatorFunction]
+    ref: str
+    extra: Any
 
 
 def function_plain_schema(
-    function: Callable[..., Any], *, validator_instance: Any | None = None, ref: str | None = None
+    function: ValidatorFunction, *, ref: str | None = None, extra: Any = None
 ) -> FunctionPlainSchema:
-    return dict_not_none(
-        type='function', mode='plain', function=function, validator_instance=validator_instance, ref=ref
-    )
+    return dict_not_none(type='function', mode='plain', function=function, ref=ref, extra=extra)
 
 
 class WithDefaultSchema(TypedDict, total=False):
@@ -504,6 +636,7 @@ class WithDefaultSchema(TypedDict, total=False):
     on_error: Literal['raise', 'omit', 'default']  # default: 'raise'
     strict: bool
     ref: str
+    extra: Any
 
 
 Omitted = object()
@@ -517,9 +650,16 @@ def with_default_schema(
     on_error: Literal['raise', 'omit', 'default'] | None = None,
     strict: bool | None = None,
     ref: str | None = None,
+    extra: Any = None,
 ) -> WithDefaultSchema:
     s = dict_not_none(
-        type='default', schema=schema, default_factory=default_factory, on_error=on_error, strict=strict, ref=ref
+        type='default',
+        schema=schema,
+        default_factory=default_factory,
+        on_error=on_error,
+        strict=strict,
+        ref=ref,
+        extra=extra,
     )
     if default is not Omitted:
         s['default'] = default
@@ -531,96 +671,65 @@ class NullableSchema(TypedDict, total=False):
     schema: Required[CoreSchema]
     strict: bool
     ref: str
+    extra: Any
 
 
-def nullable_schema(schema: CoreSchema, *, strict: bool | None = None, ref: str | None = None) -> NullableSchema:
-    return dict_not_none(type='nullable', schema=schema, strict=strict, ref=ref)
+def nullable_schema(
+    schema: CoreSchema, *, strict: bool | None = None, ref: str | None = None, extra: Any = None
+) -> NullableSchema:
+    return dict_not_none(type='nullable', schema=schema, strict=strict, ref=ref, extra=extra)
 
 
-class CustomError(TypedDict):
-    kind: str
+class CustomError(TypedDict, total=False):
+    kind: Required[str]
     message: str
-
-
-def _custom_error(kind: str | None, message: str | None) -> CustomError | None:
-    if kind is None and message is None:
-        return None
-    else:
-        # let schema validation raise the error
-        return CustomError(kind=kind, message=message)  # type: ignore
+    context: Dict[str, Union[str, int]]
 
 
 class UnionSchema(TypedDict, total=False):
     type: Required[Literal['union']]
     choices: Required[List[CoreSchema]]
-    custom_error: CustomError
+    custom_error_kind: str
+    custom_error_message: str
+    custom_error_context: Dict[str, Union[str, int, float]]
     strict: bool
     ref: str
-
-
-@overload
-def union_schema(
-    *choices: CoreSchema,
-    custom_error_kind: str,
-    custom_error_message: str,
-    strict: bool | None = None,
-    ref: str | None = None,
-) -> UnionSchema:
-    ...
-
-
-@overload
-def union_schema(*choices: CoreSchema, strict: bool | None = None, ref: str | None = None) -> UnionSchema:
-    ...
+    extra: Any
 
 
 def union_schema(
     *choices: CoreSchema,
     custom_error_kind: str | None = None,
     custom_error_message: str | None = None,
+    custom_error_context: dict[str, str | int] | None = None,
     strict: bool | None = None,
     ref: str | None = None,
+    extra: Any = None,
 ) -> UnionSchema:
     return dict_not_none(
         type='union',
         choices=choices,
-        custom_error=_custom_error(custom_error_kind, custom_error_message),
+        custom_error_kind=custom_error_kind,
+        custom_error_message=custom_error_message,
+        custom_error_context=custom_error_context,
         strict=strict,
         ref=ref,
+        extra=extra,
     )
 
 
-class TaggedUnionSchema(TypedDict):
-    type: Literal['tagged-union']
-    choices: Dict[str, CoreSchema]
-    discriminator: Union[str, List[Union[str, int]], List[List[Union[str, int]]], Callable[[Any], Optional[str]]]
-    custom_error: NotRequired[CustomError]
-    strict: NotRequired[bool]
-    ref: NotRequired[str]
-
-
-@overload
-def tagged_union_schema(
-    choices: Dict[str, CoreSchema],
-    discriminator: str | list[str | int] | list[list[str | int]] | Callable[[Any], str | None],
-    *,
-    custom_error_kind: str,
-    custom_error_message: str,
-    strict: bool | None = None,
-    ref: str | None = None,
-) -> TaggedUnionSchema:
-    ...
-
-
-@overload
-def tagged_union_schema(
-    choices: Dict[str, CoreSchema],
-    discriminator: str | list[str | int] | list[list[str | int]] | Callable[[Any], str | None],
-    *,
-    strict: bool | None = None,
-    ref: str | None = None,
-) -> TaggedUnionSchema:
-    ...
+class TaggedUnionSchema(TypedDict, total=False):
+    type: Required[Literal['tagged-union']]
+    choices: Required[Dict[str, CoreSchema]]
+    discriminator: Required[
+        Union[str, List[Union[str, int]], List[List[Union[str, int]]], Callable[[Any], Optional[str]]]
+    ]
+    custom_error_kind: str
+    custom_error_message: str
+    custom_error_context: Dict[str, Union[str, int, float]]
+    strict: bool
+    ref: str
+    extra: Any
 
 
 def tagged_union_schema(
@@ -629,27 +738,33 @@ def tagged_union_schema(
     *,
     custom_error_kind: str | None = None,
     custom_error_message: str | None = None,
+    custom_error_context: dict[str, int | str | float] | None = None,
     strict: bool | None = None,
     ref: str | None = None,
+    extra: Any = None,
 ) -> TaggedUnionSchema:
     return dict_not_none(
         type='tagged-union',
         choices=choices,
         discriminator=discriminator,
-        custom_error=_custom_error(custom_error_kind, custom_error_message),
+        custom_error_kind=custom_error_kind,
+        custom_error_message=custom_error_message,
+        custom_error_context=custom_error_context,
         strict=strict,
         ref=ref,
+        extra=extra,
     )
 
 
-class ChainSchema(TypedDict):
-    type: Literal['chain']
-    steps: List[CoreSchema]
-    ref: NotRequired[str]
+class ChainSchema(TypedDict, total=False):
+    type: Required[Literal['chain']]
+    steps: Required[List[CoreSchema]]
+    ref: str
+    extra: Any
 
 
-def chain_schema(*steps: CoreSchema, ref: str | None = None) -> ChainSchema:
-    return dict_not_none(type='chain', steps=steps, ref=ref)
+def chain_schema(*steps: CoreSchema, ref: str | None = None, extra: Any = None) -> ChainSchema:
+    return dict_not_none(type='chain', steps=steps, ref=ref, extra=extra)
 
 
 class TypedDictField(TypedDict, total=False):
@@ -676,6 +791,7 @@ class TypedDictSchema(TypedDict, total=False):
     extra_validator: CoreSchema
     return_fields_set: bool
     ref: str
+    extra: Any
     # all these values can be set via config, equivalent fields have `typed_dict_` prefix
     extra_behavior: Literal['allow', 'forbid', 'ignore']
     total: bool  # default: True
@@ -690,6 +806,7 @@ def typed_dict_schema(
     extra_validator: CoreSchema | None = None,
     return_fields_set: bool | None = None,
     ref: str | None = None,
+    extra: Any = None,
     extra_behavior: Literal['allow', 'forbid', 'ignore'] | None = None,
     total: bool | None = None,
     populate_by_name: bool | None = None,
@@ -702,6 +819,7 @@ def typed_dict_schema(
         extra_validator=extra_validator,
         return_fields_set=return_fields_set,
         ref=ref,
+        extra=extra,
         extra_behavior=extra_behavior,
         total=total,
         populate_by_name=populate_by_name,
@@ -709,14 +827,15 @@ def typed_dict_schema(
     )
 
 
-class NewClassSchema(TypedDict):
-    type: Literal['new-class']
-    cls: Type[Any]
-    schema: CoreSchema
-    call_after_init: NotRequired[str]
-    strict: NotRequired[bool]
-    ref: NotRequired[str]
-    config: NotRequired[CoreConfig]
+class NewClassSchema(TypedDict, total=False):
+    type: Required[Literal['new-class']]
+    cls: Required[Type[Any]]
+    schema: Required[CoreSchema]
+    call_after_init: str
+    strict: bool
+    ref: str
+    extra: Any
+    config: CoreConfig
 
 
 def new_class_schema(
@@ -726,6 +845,7 @@ def new_class_schema(
     call_after_init: str | None = None,
     strict: bool | None = None,
     ref: str | None = None,
+    extra: Any = None,
     config: CoreConfig | None = None,
 ) -> NewClassSchema:
     return dict_not_none(
@@ -757,6 +877,7 @@ class ArgumentsSchema(TypedDict, total=False):
     var_args_schema: CoreSchema
     var_kwargs_schema: CoreSchema
     ref: str
+    extra: Any
 
 
 def arguments_schema(
@@ -765,6 +886,7 @@ def arguments_schema(
     var_args_schema: CoreSchema | None = None,
     var_kwargs_schema: CoreSchema | None = None,
     ref: str | None = None,
+    extra: Any = None,
 ) -> ArgumentsSchema:
     return dict_not_none(
         type='arguments',
@@ -773,15 +895,17 @@ def arguments_schema(
         var_args_schema=var_args_schema,
         var_kwargs_schema=var_kwargs_schema,
         ref=ref,
+        extra=extra,
     )
 
 
-class CallSchema(TypedDict):
-    type: Literal['call']
-    arguments_schema: CoreSchema
-    function: Callable[..., Any]
-    return_schema: NotRequired[CoreSchema]
-    ref: NotRequired[str]
+class CallSchema(TypedDict, total=False):
+    type: Required[Literal['call']]
+    arguments_schema: Required[CoreSchema]
+    function: Required[Callable[..., Any]]
+    return_schema: CoreSchema
+    ref: str
+    extra: Any
 
 
 def call_schema(
@@ -790,21 +914,63 @@ def call_schema(
     *,
     return_schema: CoreSchema | None = None,
     ref: str | None = None,
+    extra: Any = None,
 ) -> CallSchema:
     return dict_not_none(
-        type='call', arguments_schema=arguments, function=function, return_schema=return_schema, ref=ref
+        type='call', arguments_schema=arguments, function=function, return_schema=return_schema, ref=ref, extra=extra
     )
 
 
-class RecursiveReferenceSchema(TypedDict):
-    type: Literal['recursive-ref']
-    schema_ref: str
+class RecursiveReferenceSchema(TypedDict, total=False):
+    type: Required[Literal['recursive-ref']]
+    schema_ref: Required[str]
 
 
 def recursive_reference_schema(schema_ref: str) -> RecursiveReferenceSchema:
     return {'type': 'recursive-ref', 'schema_ref': schema_ref}
 
 
+class CustomErrorSchema(TypedDict, total=False):
+    type: Required[Literal['custom_error']]
+    schema: Required[CoreSchema]
+    custom_error_kind: Required[str]
+    custom_error_message: str
+    custom_error_context: Dict[str, Union[str, int, float]]
+    ref: str
+    extra: Any
+
+
+def custom_error_schema(
+    schema: CoreSchema,
+    custom_error_kind: str,
+    *,
+    custom_error_message: str | None = None,
+    custom_error_context: dict[str, str | int | float] | None = None,
+    ref: str | None = None,
+    extra: Any = None,
+) -> CustomErrorSchema:
+    return dict_not_none(
+        type='custom_error',
+        schema=schema,
+        custom_error_kind=custom_error_kind,
+        custom_error_message=custom_error_message,
+        custom_error_context=custom_error_context,
+        ref=ref,
+        extra=extra,
+    )
+
+
+class JsonSchema(TypedDict, total=False):
+    type: Required[Literal['json']]
+    schema: CoreSchema
+    ref: str
+    extra: Any
+
+
+def json_schema(schema: CoreSchema | None = None, *, ref: str | None = None, extra: Any = None) -> JsonSchema:
+    return dict_not_none(type='json', schema=schema, ref=ref, extra=extra)
+
+
 CoreSchema = Union[
     AnySchema,
     NoneSchema,
@@ -825,8 +991,10 @@ CoreSchema = Union[
     TupleVariableSchema,
     SetSchema,
     FrozenSetSchema,
+    GeneratorSchema,
     DictSchema,
     FunctionSchema,
+    FunctionWrapSchema,
     FunctionPlainSchema,
     WithDefaultSchema,
     NullableSchema,
@@ -838,4 +1006,80 @@ CoreSchema = Union[
     ArgumentsSchema,
     CallSchema,
     RecursiveReferenceSchema,
+    CustomErrorSchema,
+    JsonSchema,
+]
+
+# used in _pydantic_core.pyi::PydanticKindError
+# to update this, call `pytest -k test_all_errors` and copy the output
+ErrorKind = Literal[
+    'json_invalid',
+    'json_type',
+    'recursion_loop',
+    'dict_attributes_type',
+    'missing',
+    'frozen',
+    'extra_forbidden',
+    'invalid_key',
+    'get_attribute_error',
+    'model_class_type',
+    'none_required',
+    'bool',
+    'greater_than',
+    'greater_than_equal',
+    'less_than',
+    'less_than_equal',
+    'multiple_of',
+    'finite_number',
+    'too_short',
+    'too_long',
+    'iterable_type',
+    'iteration_error',
+    'string_type',
+    'string_sub_type',
+    'string_unicode',
+    'string_too_short',
+    'string_too_long',
+    'string_pattern_mismatch',
+    'dict_type',
+    'dict_from_mapping',
+    'list_type',
+    'tuple_type',
+    'set_type',
+    'bool_type',
+    'bool_parsing',
+    'int_type',
+    'int_parsing',
+    'int_from_float',
+    'float_type',
+    'float_parsing',
+    'bytes_type',
+    'bytes_too_short',
+    'bytes_too_long',
+    'value_error',
+    'assertion_error',
+    'literal_single_error',
+    'literal_multiple_error',
+    'date_type',
+    'date_parsing',
+    'date_from_datetime_parsing',
+    'date_from_datetime_inexact',
+    'time_type',
+    'time_parsing',
+    'datetime_type',
+    'datetime_parsing',
+    'datetime_object_invalid',
+    'time_delta_type',
+    'time_delta_parsing',
+    'frozen_set_type',
+    'is_instance_of',
+    'callable_type',
+    'union_tag_invalid',
+    'union_tag_not_found',
+    'arguments_type',
+    'unexpected_keyword_argument',
+    'missing_keyword_argument',
+    'unexpected_positional_argument',
+    'missing_positional_argument',
+    'multiple_argument_values',
 ]
