@@ -4,17 +4,30 @@ use pyo3::prelude::*;
 use pyo3::types::{PyString, PyType};
 
 use crate::errors::{InputValue, LocItem, ValResult};
-use crate::input::datetime::EitherTime;
 
-use super::datetime::{EitherDate, EitherDateTime, EitherTimedelta};
+use super::datetime::{EitherDate, EitherDateTime, EitherTime, EitherTimedelta};
 use super::return_enums::{EitherBytes, EitherString};
-use super::{GenericArguments, GenericCollection, GenericMapping};
+use super::{GenericArguments, GenericCollection, GenericIterator, GenericMapping, JsonInput};
+
+pub enum InputType {
+    Python,
+    Json,
+    String,
+}
+
+impl InputType {
+    pub fn is_json(&self) -> bool {
+        matches!(self, Self::Json)
+    }
+}
 
 /// all types have three methods: `validate_*`, `strict_*`, `lax_*`
 /// the convention is to either implement:
 /// * `strict_*` & `lax_*` if they have different behavior
 /// * or, `validate_*` and `strict_*` to just call `validate_*` if the behavior for strict and lax is the same
 pub trait Input<'a>: fmt::Debug + ToPyObject {
+    fn get_type(&self) -> &'static InputType;
+
     fn as_loc_item(&self) -> LocItem;
 
     fn as_error_value(&'a self) -> InputValue<'a>;
@@ -34,9 +47,7 @@ pub trait Input<'a>: fmt::Debug + ToPyObject {
         None
     }
 
-    fn is_instance(&self, _class: &PyType) -> PyResult<bool> {
-        Ok(false)
-    }
+    fn is_instance(&self, class: &PyType, json_mask: u8) -> PyResult<bool>;
 
     fn callable(&self) -> bool {
         false
@@ -44,6 +55,8 @@ pub trait Input<'a>: fmt::Debug + ToPyObject {
 
     fn validate_args(&'a self) -> ValResult<'a, GenericArguments<'a>>;
 
+    fn parse_json(&'a self) -> ValResult<'a, JsonInput>;
+
     fn validate_str(&'a self, strict: bool) -> ValResult<EitherString<'a>> {
         if strict {
             self.strict_str()
@@ -126,16 +139,16 @@ pub trait Input<'a>: fmt::Debug + ToPyObject {
         self.validate_dict(strict)
     }
 
-    fn validate_list(&'a self, strict: bool) -> ValResult<GenericCollection<'a>> {
-        if strict {
+    fn validate_list(&'a self, strict: bool, allow_any_iter: bool) -> ValResult<GenericCollection<'a>> {
+        if strict && !allow_any_iter {
             self.strict_list()
         } else {
-            self.lax_list()
+            self.lax_list(allow_any_iter)
         }
     }
     fn strict_list(&'a self) -> ValResult<GenericCollection<'a>>;
     #[cfg_attr(has_no_coverage, no_coverage)]
-    fn lax_list(&'a self) -> ValResult<GenericCollection<'a>> {
+    fn lax_list(&'a self, _allow_any_iter: bool) -> ValResult<GenericCollection<'a>> {
         self.strict_list()
     }
 
@@ -178,6 +191,8 @@ pub trait Input<'a>: fmt::Debug + ToPyObject {
         self.strict_frozenset()
     }
 
+    fn validate_iter(&self) -> ValResult<GenericIterator>;
+
     fn validate_date(&self, strict: bool) -> ValResult<EitherDate> {
         if strict {
             self.strict_date()
