@@ -2,6 +2,7 @@ use std::borrow::Cow;
 use std::str::from_utf8;
 
 use pyo3::exceptions::PyAttributeError;
+use pyo3::once_cell::GILOnceCell;
 use pyo3::prelude::*;
 use pyo3::types::{
     PyBool, PyByteArray, PyBytes, PyDate, PyDateTime, PyDelta, PyDict, PyFrozenSet, PyIterator, PyList, PyMapping,
@@ -9,7 +10,7 @@ use pyo3::types::{
 };
 #[cfg(not(PyPy))]
 use pyo3::types::{PyDictItems, PyDictKeys, PyDictValues};
-use pyo3::{intern, AsPyPointer};
+use pyo3::{intern, AsPyPointer, PyTypeInfo};
 
 use crate::errors::{py_err_string, ErrorKind, InputValue, LocItem, ValError, ValResult};
 
@@ -18,35 +19,47 @@ use super::datetime::{
     float_as_duration, float_as_time, int_as_datetime, int_as_duration, int_as_time, EitherDate, EitherDateTime,
     EitherTime,
 };
-use super::shared::{float_as_int, int_as_bool, str_as_bool, str_as_int};
+use super::input_abstract::InputType;
+use super::shared::{float_as_int, int_as_bool, map_json_err, str_as_bool, str_as_int};
 use super::{
     py_string_str, repr_string, EitherBytes, EitherString, EitherTimedelta, GenericArguments, GenericCollection,
-    GenericMapping, Input, PyArgs,
+    GenericIterator, GenericMapping, Input, JsonInput, PyArgs,
 };
 
-#[cfg(not(PyPy))]
-macro_rules! extract_gen_dict {
-    ($type:ty, $obj:ident) => {{
-        let map_err = |_| ValError::new(ErrorKind::IterationError, $obj);
-        if let Ok(iterator) = $obj.cast_as::<PyIterator>() {
-            let vec = iterator.collect::<PyResult<Vec<_>>>().map_err(map_err)?;
-            Some(<$type>::new($obj.py(), vec))
-        } else if let Ok(dict_keys) = $obj.cast_as::<PyDictKeys>() {
-            let vec = dict_keys.iter()?.collect::<PyResult<Vec<_>>>().map_err(map_err)?;
-            Some(<$type>::new($obj.py(), vec))
-        } else if let Ok(dict_values) = $obj.cast_as::<PyDictValues>() {
-            let vec = dict_values.iter()?.collect::<PyResult<Vec<_>>>().map_err(map_err)?;
-            Some(<$type>::new($obj.py(), vec))
-        } else if let Ok(dict_items) = $obj.cast_as::<PyDictItems>() {
-            let vec = dict_items.iter()?.collect::<PyResult<Vec<_>>>().map_err(map_err)?;
-            Some(<$type>::new($obj.py(), vec))
+/// Extract generators and deques into a `GenericCollection`
+macro_rules! extract_shared_iter {
+    ($type:ty, $obj:ident) => {
+        if $obj.cast_as::<PyIterator>().is_ok() {
+            Some($obj.into())
+        } else if is_deque($obj) {
+            Some($obj.into())
         } else {
             None
         }
-    }};
+    };
+}
+
+/// Extract dict keys, values and items into a `GenericCollection`, not available on PyPy
+#[cfg(not(PyPy))]
+macro_rules! extract_dict_iter {
+    ($obj:ident) => {
+        if $obj.is_instance_of::<PyDictKeys>().unwrap_or(false) {
+            Some($obj.into())
+        } else if $obj.is_instance_of::<PyDictValues>().unwrap_or(false) {
+            Some($obj.into())
+        } else if $obj.is_instance_of::<PyDictItems>().unwrap_or(false) {
+            Some($obj.into())
+        } else {
+            None
+        }
+    };
 }
 
 impl<'a> Input<'a> for PyAny {
+    fn get_type(&self) -> &'static InputType {
+        &InputType::Python
+    }
+
     fn as_loc_item(&self) -> LocItem {
         if let Ok(py_str) = self.cast_as::<PyString>() {
             py_str.to_string_lossy().as_ref().into()
@@ -80,7 +93,7 @@ impl<'a> Input<'a> for PyAny {
         self.getattr(name).ok()
     }
 
-    fn is_instance(&self, class: &PyType) -> PyResult<bool> {
+    fn is_instance(&self, class: &PyType, _json_mask: u8) -> PyResult<bool> {
         self.is_instance(class)
     }
 
@@ -114,31 +127,54 @@ impl<'a> Input<'a> for PyAny {
         }
     }
 
+    fn parse_json(&'a self) -> ValResult<'a, JsonInput> {
+        if let Ok(py_bytes) = self.cast_as::<PyBytes>() {
+            serde_json::from_slice(py_bytes.as_bytes()).map_err(|e| map_json_err(self, e))
+        } else if let Ok(py_str) = self.cast_as::<PyString>() {
+            let str = py_str.to_str()?;
+            serde_json::from_str(str).map_err(|e| map_json_err(self, e))
+        } else if let Ok(py_byte_array) = self.cast_as::<PyByteArray>() {
+            serde_json::from_slice(unsafe { py_byte_array.as_bytes() }).map_err(|e| map_json_err(self, e))
+        } else {
+            Err(ValError::new(ErrorKind::JsonType, self))
+        }
+    }
+
     fn strict_str(&'a self) -> ValResult<EitherString<'a>> {
         if let Ok(py_str) = self.cast_as::<PyString>() {
-            Ok(py_str.into())
+            if is_builtin_str(py_str) {
+                Ok(py_str.into())
+            } else {
+                Err(ValError::new(ErrorKind::StringSubType, self))
+            }
         } else {
-            Err(ValError::new(ErrorKind::StrType, self))
+            Err(ValError::new(ErrorKind::StringType, self))
         }
     }
 
     fn lax_str(&'a self) -> ValResult<EitherString<'a>> {
         if let Ok(py_str) = self.cast_as::<PyString>() {
-            Ok(py_str.into())
+            if is_builtin_str(py_str) {
+                Ok(py_str.into())
+            } else {
+                // force to a rust string to make sure behaviour is consistent whether or not we go via a
+                // rust string in StrConstrainedValidator - e.g. to_lower
+                Ok(py_string_str(py_str)?.into())
+            }
         } else if let Ok(bytes) = self.cast_as::<PyBytes>() {
             let str = match from_utf8(bytes.as_bytes()) {
                 Ok(s) => s,
-                Err(_) => return Err(ValError::new(ErrorKind::StrUnicode, self)),
+                Err(_) => return Err(ValError::new(ErrorKind::StringUnicode, self)),
             };
             Ok(str.into())
         } else if let Ok(py_byte_array) = self.cast_as::<PyByteArray>() {
             let str = match from_utf8(unsafe { py_byte_array.as_bytes() }) {
                 Ok(s) => s,
-                Err(_) => return Err(ValError::new(ErrorKind::StrUnicode, self)),
+                Err(_) => return Err(ValError::new(ErrorKind::StringUnicode, self)),
             };
             Ok(str.into())
         } else {
-            Err(ValError::new(ErrorKind::StrType, self))
+            Err(ValError::new(ErrorKind::StringType, self))
         }
     }
 
@@ -286,29 +322,32 @@ impl<'a> Input<'a> for PyAny {
     }
 
     #[cfg(not(PyPy))]
-    fn lax_list(&'a self) -> ValResult<GenericCollection<'a>> {
+    fn lax_list(&'a self, allow_any_iter: bool) -> ValResult<GenericCollection<'a>> {
         if let Ok(list) = self.cast_as::<PyList>() {
             Ok(list.into())
         } else if let Ok(tuple) = self.cast_as::<PyTuple>() {
             Ok(tuple.into())
-        } else if let Some(list) = extract_gen_dict!(PyList, self) {
-            Ok(list.into())
+        } else if let Some(collection) = extract_dict_iter!(self) {
+            Ok(collection)
+        } else if allow_any_iter && self.iter().is_ok() {
+            Ok(self.into())
+        } else if let Some(collection) = extract_shared_iter!(PyList, self) {
+            Ok(collection)
         } else {
             Err(ValError::new(ErrorKind::ListType, self))
         }
     }
 
     #[cfg(PyPy)]
-    fn lax_list(&'a self) -> ValResult<GenericCollection<'a>> {
+    fn lax_list(&'a self, allow_any_iter: bool) -> ValResult<GenericCollection<'a>> {
         if let Ok(list) = self.cast_as::<PyList>() {
             Ok(list.into())
         } else if let Ok(tuple) = self.cast_as::<PyTuple>() {
             Ok(tuple.into())
-        } else if let Ok(iterator) = self.cast_as::<PyIterator>() {
-            let vec = iterator
-                .collect::<PyResult<Vec<_>>>()
-                .map_err(|_| ValError::new(ErrorKind::IterationError, self))?;
-            Ok(PyList::new(self.py(), vec).into())
+        } else if allow_any_iter && self.iter().is_ok() {
+            Ok(self.into())
+        } else if let Some(collection) = extract_shared_iter!(PyList, self) {
+            Ok(collection)
         } else {
             Err(ValError::new(ErrorKind::ListType, self))
         }
@@ -328,8 +367,10 @@ impl<'a> Input<'a> for PyAny {
             Ok(tuple.into())
         } else if let Ok(list) = self.cast_as::<PyList>() {
             Ok(list.into())
-        } else if let Some(tuple) = extract_gen_dict!(PyTuple, self) {
-            Ok(tuple.into())
+        } else if let Some(collection) = extract_dict_iter!(self) {
+            Ok(collection)
+        } else if let Some(collection) = extract_shared_iter!(PyTuple, self) {
+            Ok(collection)
         } else {
             Err(ValError::new(ErrorKind::TupleType, self))
         }
@@ -341,11 +382,8 @@ impl<'a> Input<'a> for PyAny {
             Ok(tuple.into())
         } else if let Ok(list) = self.cast_as::<PyList>() {
             Ok(list.into())
-        } else if let Ok(iterator) = self.cast_as::<PyIterator>() {
-            let vec = iterator
-                .collect::<PyResult<Vec<_>>>()
-                .map_err(|_| ValError::new(ErrorKind::IterationError, self))?;
-            Ok(PyTuple::new(self.py(), vec).into())
+        } else if let Some(collection) = extract_shared_iter!(PyTuple, self) {
+            Ok(collection)
         } else {
             Err(ValError::new(ErrorKind::TupleType, self))
         }
@@ -369,8 +407,10 @@ impl<'a> Input<'a> for PyAny {
             Ok(tuple.into())
         } else if let Ok(frozen_set) = self.cast_as::<PyFrozenSet>() {
             Ok(frozen_set.into())
-        } else if let Some(tuple) = extract_gen_dict!(PyTuple, self) {
-            Ok(tuple.into())
+        } else if let Some(collection) = extract_dict_iter!(self) {
+            Ok(collection)
+        } else if let Some(collection) = extract_shared_iter!(PyTuple, self) {
+            Ok(collection)
         } else {
             Err(ValError::new(ErrorKind::SetType, self))
         }
@@ -386,11 +426,8 @@ impl<'a> Input<'a> for PyAny {
             Ok(tuple.into())
         } else if let Ok(frozen_set) = self.cast_as::<PyFrozenSet>() {
             Ok(frozen_set.into())
-        } else if let Ok(iterator) = self.cast_as::<PyIterator>() {
-            let vec = iterator
-                .collect::<PyResult<Vec<_>>>()
-                .map_err(|_| ValError::new(ErrorKind::IterationError, self))?;
-            Ok(PyTuple::new(self.py(), vec).into())
+        } else if let Some(collection) = extract_shared_iter!(PyTuple, self) {
+            Ok(collection)
         } else {
             Err(ValError::new(ErrorKind::SetType, self))
         }
@@ -414,8 +451,10 @@ impl<'a> Input<'a> for PyAny {
             Ok(list.into())
         } else if let Ok(tuple) = self.cast_as::<PyTuple>() {
             Ok(tuple.into())
-        } else if let Some(tuple) = extract_gen_dict!(PyTuple, self) {
-            Ok(tuple.into())
+        } else if let Some(collection) = extract_dict_iter!(self) {
+            Ok(collection)
+        } else if let Some(collection) = extract_shared_iter!(PyTuple, self) {
+            Ok(collection)
         } else {
             Err(ValError::new(ErrorKind::FrozenSetType, self))
         }
@@ -431,16 +470,21 @@ impl<'a> Input<'a> for PyAny {
             Ok(list.into())
         } else if let Ok(tuple) = self.cast_as::<PyTuple>() {
             Ok(tuple.into())
-        } else if let Ok(iterator) = self.cast_as::<PyIterator>() {
-            let vec = iterator
-                .collect::<PyResult<Vec<_>>>()
-                .map_err(|_| ValError::new(ErrorKind::IterationError, self))?;
-            Ok(PyTuple::new(self.py(), vec).into())
+        } else if let Some(collection) = extract_shared_iter!(PyTuple, self) {
+            Ok(collection)
         } else {
             Err(ValError::new(ErrorKind::FrozenSetType, self))
         }
     }
 
+    fn validate_iter(&self) -> ValResult<GenericIterator> {
+        if self.iter().is_ok() {
+            Ok(self.into())
+        } else {
+            Err(ValError::new(ErrorKind::IterableType, self))
+        }
+    }
+
     fn strict_date(&self) -> ValResult<EitherDate> {
         if self.cast_as::<PyDateTime>().is_ok() {
             // have to check if it's a datetime first, otherwise the line below converts to a date
@@ -500,7 +544,7 @@ impl<'a> Input<'a> for PyAny {
         if let Ok(dt) = self.cast_as::<PyDateTime>() {
             Ok(dt.into())
         } else {
-            Err(ValError::new(ErrorKind::DateTimeType, self))
+            Err(ValError::new(ErrorKind::DatetimeType, self))
         }
     }
 
@@ -513,7 +557,7 @@ impl<'a> Input<'a> for PyAny {
         } else if let Ok(py_bytes) = self.cast_as::<PyBytes>() {
             bytes_as_datetime(self, py_bytes.as_bytes())
         } else if self.cast_as::<PyBool>().is_ok() {
-            Err(ValError::new(ErrorKind::DateTimeType, self))
+            Err(ValError::new(ErrorKind::DatetimeType, self))
         } else if let Ok(int) = self.extract::<i64>() {
             int_as_datetime(self, int, 0)
         } else if let Ok(float) = self.extract::<f64>() {
@@ -521,7 +565,7 @@ impl<'a> Input<'a> for PyAny {
         } else if let Ok(date) = self.cast_as::<PyDate>() {
             Ok(date_as_datetime(date)?)
         } else {
-            Err(ValError::new(ErrorKind::DateTimeType, self))
+            Err(ValError::new(ErrorKind::DatetimeType, self))
         }
     }
 
@@ -623,3 +667,22 @@ fn maybe_as_string(v: &PyAny, unicode_error: ErrorKind) -> ValResult<Option<Cow<
         Ok(None)
     }
 }
+
+static DEQUE_TYPE: GILOnceCell<Py<PyType>> = GILOnceCell::new();
+
+fn is_deque(v: &PyAny) -> bool {
+    let py = v.py();
+    let deque_type = DEQUE_TYPE
+        .get_or_init(py, || import_type(py, "collections", "deque").unwrap())
+        .as_ref(py);
+    v.is_instance(deque_type).unwrap_or(false)
+}
+
+fn import_type(py: Python, module: &str, attr: &str) -> PyResult<Py<PyType>> {
+    let obj = py.import(module)?.getattr(attr)?;
+    Ok(obj.cast_as::<PyType>()?.into())
+}
+
+fn is_builtin_str(py_str: &PyString) -> bool {
+    py_str.get_type().is(PyString::type_object(py_str.py()))
+}
