@@ -2,6 +2,15 @@ use crate::errors::{ErrorKind, ValError, ValResult};
 
 use super::Input;
 
+pub fn map_json_err<'a>(input: &'a impl Input<'a>, error: serde_json::Error) -> ValError<'a> {
+    ValError::new(
+        ErrorKind::JsonInvalid {
+            error: error.to_string(),
+        },
+        input,
+    )
+}
+
 #[inline]
 pub fn str_as_bool<'a>(input: &'a impl Input<'a>, str: &str) -> ValResult<'a, bool> {
     if str == "0"
@@ -48,17 +57,8 @@ pub fn str_as_int<'s, 'l>(input: &'s impl Input<'s>, str: &'l str) -> ValResult<
 }
 
 pub fn float_as_int<'a>(input: &'a impl Input<'a>, float: f64) -> ValResult<'a, i64> {
-    if float == f64::INFINITY {
-        Err(ValError::new(ErrorKind::IntNan { nan_value: "infinity" }, input))
-    } else if float == f64::NEG_INFINITY {
-        Err(ValError::new(
-            ErrorKind::IntNan {
-                nan_value: "negative infinity",
-            },
-            input,
-        ))
-    } else if float.is_nan() {
-        Err(ValError::new(ErrorKind::IntNan { nan_value: "NaN" }, input))
+    if float == f64::INFINITY || float == f64::NEG_INFINITY || float.is_nan() {
+        Err(ValError::new(ErrorKind::FiniteNumber, input))
     } else if float % 1.0 != 0.0 {
         Err(ValError::new(ErrorKind::IntFromFloat, input))
     } else {
