@@ -6,11 +6,12 @@ use pyo3::types::{PyDict, PyList, PyString};
 
 use ahash::AHashSet;
 
-use crate::build_tools::{py_error, SchemaDict};
+use crate::build_tools::{py_err, SchemaDict};
 use crate::errors::{ErrorKind, ValError, ValResult};
 use crate::input::Input;
 use crate::recursion_guard::{NoHashSet, RecursionGuard};
 
+use super::none::NoneValidator;
 use super::{BuildContext, BuildValidator, CombinedValidator, Extra, Validator};
 
 #[derive(Debug)]
@@ -21,19 +22,20 @@ impl BuildValidator for LiteralBuilder {
 
     fn build(
         schema: &PyDict,
-        _config: Option<&PyDict>,
-        _build_context: &mut BuildContext,
+        config: Option<&PyDict>,
+        build_context: &mut BuildContext,
     ) -> PyResult<CombinedValidator> {
         let expected: &PyList = schema.get_as_req(intern!(schema.py(), "expected"))?;
         if expected.is_empty() {
-            return py_error!(r#""expected" should have length > 0"#);
+            return py_err!(r#""expected" should have length > 0"#);
         } else if expected.len() == 1 {
             let first = expected.get_item(0)?;
             if let Ok(py_str) = first.cast_as::<PyString>() {
                 return Ok(LiteralSingleStringValidator::new(py_str.to_str()?.to_string()).into());
-            }
-            if let Ok(int) = first.extract::<i64>() {
+            } else if let Ok(int) = first.extract::<i64>() {
                 return Ok(LiteralSingleIntValidator::new(int).into());
+            } else if first.is_none() {
+                return NoneValidator::build(schema, config, build_context);
             }
         }
 
