@@ -3,16 +3,15 @@ use std::fmt::Debug;
 use enum_dispatch::enum_dispatch;
 
 use ahash::AHashSet;
-use pyo3::exceptions::PyTypeError;
 use pyo3::intern;
 use pyo3::once_cell::GILOnceCell;
 use pyo3::prelude::*;
-use pyo3::types::{PyAny, PyByteArray, PyBytes, PyDict, PyString};
+use pyo3::types::{PyAny, PyDict};
 
 use crate::build_context::{extract_used_refs, BuildContext};
-use crate::build_tools::{py_error, SchemaDict, SchemaError};
-use crate::errors::{ErrorKind, ValError, ValLineError, ValResult, ValidationError};
-use crate::input::{Input, JsonInput};
+use crate::build_tools::{py_err, py_error_type, SchemaDict, SchemaError};
+use crate::errors::{ValError, ValResult, ValidationError};
+use crate::input::Input;
 use crate::questions::{Answers, Question};
 use crate::recursion_guard::RecursionGuard;
 
@@ -23,14 +22,17 @@ mod bytes;
 mod call;
 mod callable;
 mod chain;
+mod custom_error;
 mod date;
 mod datetime;
 mod dict;
 mod float;
 mod frozenset;
 mod function;
+mod generator;
 mod int;
 mod is_instance;
+mod json;
 mod list;
 mod literal;
 mod new_class;
@@ -81,7 +83,14 @@ impl SchemaValidator {
         let mut validator = build_validator(schema, config, &mut build_context)?;
         validator.complete(&build_context)?;
         let slots = build_context.into_slots()?;
-        let title = validator.get_name().into_py(py);
+        let config_title = match config {
+            Some(c) => c.get_item("title"),
+            None => None,
+        };
+        let title = match config_title {
+            Some(t) => t.into_py(py),
+            None => validator.get_name().into_py(py),
+        };
         Ok(Self {
             validator,
             slots,
@@ -129,7 +138,8 @@ impl SchemaValidator {
         ) {
             Ok(_) => Ok(true),
             Err(ValError::InternalErr(err)) => Err(err),
-            _ => Ok(false),
+            Err(ValError::Omit) => Err(ValidationError::omit_error()),
+            Err(ValError::LineErrors(_)) => Ok(false),
         }
     }
 
@@ -140,7 +150,7 @@ impl SchemaValidator {
         strict: Option<bool>,
         context: Option<&PyAny>,
     ) -> PyResult<PyObject> {
-        match parse_json(input)? {
+        match input.parse_json() {
             Ok(input) => {
                 let r = self.validator.validate(
                     py,
@@ -151,11 +161,7 @@ impl SchemaValidator {
                 );
                 r.map_err(|e| self.prepare_validation_err(py, e))
             }
-            Err(e) => {
-                let line_err = ValLineError::new(ErrorKind::InvalidJson { error: e.to_string() }, input);
-                let err = ValError::LineErrors(vec![line_err]);
-                Err(self.prepare_validation_err(py, err))
-            }
+            Err(err) => Err(self.prepare_validation_err(py, err)),
         }
     }
 
@@ -166,7 +172,7 @@ impl SchemaValidator {
         strict: Option<bool>,
         context: Option<&PyAny>,
     ) -> PyResult<bool> {
-        match parse_json(input)? {
+        match input.parse_json() {
             Ok(input) => {
                 match self.validator.validate(
                     py,
@@ -177,7 +183,8 @@ impl SchemaValidator {
                 ) {
                     Ok(_) => Ok(true),
                     Err(ValError::InternalErr(err)) => Err(err),
-                    _ => Ok(false),
+                    Err(ValError::Omit) => Err(ValidationError::omit_error()),
+                    Err(ValError::LineErrors(_)) => Ok(false),
                 }
             }
             Err(_) => Ok(false),
@@ -236,7 +243,7 @@ impl SchemaValidator {
 
         let validator = match build_validator(self_schema, None, &mut build_context) {
             Ok(v) => v,
-            Err(err) => return Err(SchemaError::new_err(format!("Error building self-schema:\n  {}", err))),
+            Err(err) => return py_err!("Error building self-schema:\n  {}", err),
         };
         Ok(Self {
             validator,
@@ -247,21 +254,7 @@ impl SchemaValidator {
     }
 
     fn prepare_validation_err(&self, py: Python, error: ValError) -> PyErr {
-        ValidationError::from_val_error(py, self.title.clone_ref(py), error)
-    }
-}
-
-fn parse_json(input: &PyAny) -> PyResult<serde_json::Result<JsonInput>> {
-    if let Ok(py_bytes) = input.cast_as::<PyBytes>() {
-        Ok(serde_json::from_slice(py_bytes.as_bytes()))
-    } else if let Ok(py_str) = input.cast_as::<PyString>() {
-        let str = py_str.to_str()?;
-        Ok(serde_json::from_str(str))
-    } else if let Ok(py_byte_array) = input.cast_as::<PyByteArray>() {
-        Ok(serde_json::from_slice(unsafe { py_byte_array.as_bytes() }))
-    } else {
-        let input_type = input.get_type().name().unwrap_or("unknown");
-        py_error!(PyTypeError; "JSON input should be str, bytes or bytearray, not {}", input_type)
+        ValidationError::from_val_error(py, self.title.clone_ref(py), error, None)
     }
 }
 
@@ -297,7 +290,7 @@ fn build_specific_validator<'a, T: BuildValidator>(
     }
 
     T::build(schema_dict, config, build_context)
-        .map_err(|err| SchemaError::new_err(format!("Error building \"{}\" validator:\n  {}", val_type, err)))
+        .map_err(|err| py_error_type!("Error building \"{}\" validator:\n  {}", val_type, err))
 }
 
 // macro to build the match statement for validator selection
@@ -308,7 +301,7 @@ macro_rules! validator_match {
                 <$validator>::EXPECTED_TYPE => build_specific_validator::<$validator>($type, $dict, $config, $build_context),
             )+
             _ => {
-                return py_error!(r#"Unknown schema type: "{}""#, $type)
+                return py_err!(r#"Unknown schema type: "{}""#, $type)
             },
         }
     };
@@ -384,11 +377,17 @@ pub fn build_validator<'a>(
         with_default::WithDefaultValidator,
         // chain validators
         chain::ChainValidator,
+        // generator validators
+        generator::GeneratorValidator,
+        // custom error
+        custom_error::CustomErrorValidator,
+        // json data
+        json::JsonValidator,
     )
 }
 
 /// More (mostly immutable) data to pass between validators, should probably be class `Context`,
-/// but that would confuse it with context as per samuelcolvin/pydantic#1549
+/// but that would confuse it with context as per pydantic/pydantic#1549
 #[derive(Debug, Default)]
 pub struct Extra<'a> {
     /// This is used as the `data` kwargs to validator functions, it also represents the current model
@@ -433,8 +432,8 @@ pub enum CombinedValidator {
     TaggedUnion(union::TaggedUnionValidator),
     // nullables
     Nullable(nullable::NullableValidator),
-    // model classes
-    ModelClass(new_class::NewClassValidator),
+    // create new model classes
+    NewClass(new_class::NewClassValidator),
     // strings
     Str(string::StrValidator),
     StrConstrained(string::StrConstrainedValidator),
@@ -496,6 +495,12 @@ pub enum CombinedValidator {
     WithDefault(with_default::WithDefaultValidator),
     // chain validators
     Chain(chain::ChainValidator),
+    // generator validators
+    Generator(generator::GeneratorValidator),
+    // custom error
+    CustomError(custom_error::CustomErrorValidator),
+    // json data
+    Json(json::JsonValidator),
 }
 
 /// This trait must be implemented by all validators, it allows various validators to be accessed consistently,
