@@ -10,7 +10,13 @@ from typing import Dict, FrozenSet, List, Optional, Set, Union
 
 import pytest
 
-from pydantic_core import PydanticValueError, SchemaValidator, ValidationError, ValidationError as CoreValidationError
+from pydantic_core import (
+    PydanticCustomError,
+    SchemaValidator,
+    ValidationError,
+    ValidationError as CoreValidationError,
+    core_schema,
+)
 
 if os.getenv('BENCHMARK_VS_PYDANTIC'):
     try:
@@ -206,12 +212,24 @@ def test_small_class_core_model(benchmark):
 
 @pytest.mark.benchmark(group='string')
 def test_core_string_lax(benchmark):
-    validator = SchemaValidator({'type': 'str'})
+    validator = SchemaValidator(core_schema.string_schema())
     input_str = 'Hello ' * 20
 
+    assert validator.validate_python(input_str) == input_str
+
     benchmark(validator.validate_python, input_str)
 
 
+@pytest.mark.benchmark(group='string')
+def test_core_string_strict(benchmark):
+    validator = SchemaValidator(core_schema.string_schema(strict=True))
+    input_str = 'Hello ' * 20
+
+    assert validator.validate_python(input_str) == input_str
+
+    benchmark(validator.validate_python, 'foo')
+
+
 @pytest.fixture
 def recursive_model_data():
     data = {'width': -1}
@@ -805,7 +823,7 @@ def test_raise_error_value_error(benchmark):
 @pytest.mark.benchmark(group='raise-error')
 def test_raise_error_custom(benchmark):
     def f(input_value, **kwargs):
-        raise PydanticValueError('my_error', 'this is a custom error {foo}', {'foo': 'FOOBAR'})
+        raise PydanticCustomError('my_error', 'this is a custom error {foo}', {'foo': 'FOOBAR'})
 
     v = SchemaValidator({'type': 'function', 'mode': 'plain', 'function': f})
 
@@ -954,3 +972,58 @@ def test_chain_nested_functions(benchmark):
     assert validator.validate_python('42.42') == Decimal('84.84')
 
     benchmark(validator.validate_python, '42.42')
+
+
+def validate_yield(iterable, validator):
+    for item in iterable:
+        yield validator(item)
+
+
+def generator_gen_python(v, *, validator, **_kwargs):
+    try:
+        iterable = iter(v)
+    except TypeError:
+        raise PydanticCustomError('iterable_type', 'Input should be a valid iterable')
+    return validate_yield(iterable, validator)
+
+
+@pytest.mark.benchmark(group='generator')
+def test_generator_python(benchmark):
+    schema = core_schema.function_wrap_schema(generator_gen_python, {'type': 'int'})
+    v = SchemaValidator(schema)
+    input_value = tuple(range(100))
+
+    assert sum(v.validate_python(input_value)) == 4950
+
+    benchmark(v.validate_python, input_value)
+
+
+def generator_gen_rust(v, *, validator, **_kwargs):
+    try:
+        generator = iter(v)
+    except TypeError:
+        raise PydanticCustomError('generator_type', 'Input should be a valid generator')
+    return validator.iter(generator)
+
+
+@pytest.mark.benchmark(group='generator')
+def test_generator_rust(benchmark):
+    schema = {'type': 'generator', 'items_schema': {'type': 'int'}}
+    v = SchemaValidator(schema)
+    input_value = tuple(range(100))
+
+    assert sum(v.validate_python(input_value)) == 4950
+
+    benchmark(v.validate_python, input_value)
+
+
+@pytest.mark.benchmark(group='isinstance-json')
+def test_isinstance_json(benchmark):
+    validator = SchemaValidator(core_schema.is_instance_schema(str, json_types={'str'}))
+    assert validator.isinstance_json('"foo"') is True
+    assert validator.isinstance_json('123') is False
+
+    @benchmark
+    def t():
+        validator.isinstance_json('"foo"')
+        validator.isinstance_json('123')
