@@ -5,7 +5,7 @@ from typing import Type
 
 import pytest
 
-from pydantic_core import PydanticValueError, SchemaError, SchemaValidator, ValidationError
+from pydantic_core import SchemaError, SchemaValidator, ValidationError
 
 from ..conftest import plain_repr
 
@@ -52,7 +52,7 @@ def test_function_before_error():
         v.validate_python('12345')
     assert exc_info.value.errors() == [
         {
-            'kind': 'too_long',
+            'kind': 'string_too_long',
             'loc': [],
             'message': 'String should have at most 5 characters',
             'input_value': '12345x',
@@ -81,7 +81,7 @@ def test_function_before_error_model():
         v.validate_python({'my_field': '12345'})
     assert exc_info.value.errors() == [
         {
-            'kind': 'too_long',
+            'kind': 'string_too_long',
             'loc': ['my_field'],
             'message': 'String should have at most 5 characters',
             'input_value': '12345x',
@@ -92,7 +92,7 @@ def test_function_before_error_model():
 
 def test_function_wrap():
     def f(input_value, *, validator, **kwargs):
-        return validator(input_value) + ' Changed'
+        return validator(input_value=input_value) + ' Changed'
 
     v = SchemaValidator({'type': 'function', 'mode': 'wrap', 'function': f, 'schema': {'type': 'str'}})
 
@@ -119,16 +119,21 @@ def test_function_wrap_str():
 
 
 def test_function_wrap_not_callable():
-    with pytest.raises(SchemaError, match='function -> function\n  Input should be callable'):
+    with pytest.raises(SchemaError, match='function-wrap -> function\n  Input should be callable'):
         SchemaValidator({'type': 'function', 'mode': 'wrap', 'function': [], 'schema': {'type': 'str'}})
 
-    with pytest.raises(SchemaError, match='function -> function\n  Field required'):
+    with pytest.raises(SchemaError, match='function-wrap -> function\n  Field required'):
         SchemaValidator({'type': 'function', 'mode': 'wrap', 'schema': {'type': 'str'}})
 
 
 def test_wrap_error():
     def f(input_value, *, validator, **kwargs):
-        return validator(input_value) * 2
+        try:
+            return validator(input_value) * 2
+        except ValidationError as e:
+            assert e.title == 'ValidatorCallable'
+            assert str(e).startswith('1 validation error for ValidatorCallable\n')
+            raise e
 
     v = SchemaValidator({'type': 'function', 'mode': 'wrap', 'function': f, 'schema': {'type': 'int'}})
 
@@ -145,6 +150,36 @@ def test_wrap_error():
     ]
 
 
+def test_function_wrap_location():
+    def f(input_value, *, validator, **kwargs):
+        return validator(input_value, outer_location='foo') + 2
+
+    v = SchemaValidator({'type': 'function', 'mode': 'wrap', 'function': f, 'schema': {'type': 'int'}})
+
+    assert v.validate_python(4) == 6
+    with pytest.raises(ValidationError) as exc_info:
+        v.validate_python('wrong')
+    # insert_assert(exc_info.value.errors())
+    assert exc_info.value.errors() == [
+        {
+            'kind': 'int_parsing',
+            'loc': ['foo'],
+            'message': 'Input should be a valid integer, unable to parse string as an integer',
+            'input_value': 'wrong',
+        }
+    ]
+
+
+def test_function_wrap_invalid_location():
+    def f(input_value, *, validator, **kwargs):
+        return validator(input_value, ('4',)) + 2
+
+    v = SchemaValidator({'type': 'function', 'mode': 'wrap', 'function': f, 'schema': {'type': 'int'}})
+
+    with pytest.raises(TypeError, match='^ValidatorCallable outer_location must be a str or int$'):
+        v.validate_python(4)
+
+
 def test_wrong_mode():
     with pytest.raises(SchemaError, match='function -> mode\n  Input should be one of'):
         SchemaValidator({'type': 'function', 'mode': 'foobar', 'schema': {'type': 'str'}})
@@ -287,7 +322,7 @@ def test_class_with_validator():
         v.validate_python(True)
 
     assert exc_info.value.errors() == [
-        {'kind': 'str_type', 'loc': [], 'message': 'Input should be a valid string', 'input_value': True}
+        {'kind': 'string_type', 'loc': [], 'message': 'Input should be a valid string', 'input_value': True}
     ]
 
 
@@ -354,118 +389,3 @@ def test_raise_type_error():
 
     with pytest.raises(TypeError, match='^foobar$'):
         v.validate_python('input value')
-
-
-def test_pydantic_value_error():
-    e = PydanticValueError(
-        'my_error', 'this is a custom error {missed} {foo} {bar} {spam}', {'foo': 'X', 'bar': 42, 'spam': []}
-    )
-    assert e.message() == 'this is a custom error {missed} X 42 []'
-    assert e.message_template == 'this is a custom error {missed} {foo} {bar} {spam}'
-    assert e.kind == 'my_error'
-    assert e.context == {'foo': 'X', 'bar': 42, 'spam': []}
-    assert str(e) == 'this is a custom error {missed} X 42 []'
-    assert repr(e) == (
-        "this is a custom error {missed} X 42 [] [kind=my_error, context={'foo': 'X', 'bar': 42, 'spam': []}]"
-    )
-
-
-def test_pydantic_value_error_none():
-    e = PydanticValueError('my_error', 'this is a custom error {missed}')
-    assert e.message() == 'this is a custom error {missed}'
-    assert e.message_template == 'this is a custom error {missed}'
-    assert e.kind == 'my_error'
-    assert e.context is None
-    assert str(e) == 'this is a custom error {missed}'
-    assert repr(e) == 'this is a custom error {missed} [kind=my_error, context=None]'
-
-
-def test_pydantic_value_error_usage():
-    def f(input_value, **kwargs):
-        raise PydanticValueError('my_error', 'this is a custom error {foo} {bar}', {'foo': 'FOOBAR', 'bar': 42})
-
-    v = SchemaValidator({'type': 'function', 'mode': 'plain', 'function': f})
-
-    with pytest.raises(ValidationError) as exc_info:
-        v.validate_python(42)
-
-    assert exc_info.value.errors() == [
-        {
-            'kind': 'my_error',
-            'loc': [],
-            'message': 'this is a custom error FOOBAR 42',
-            'input_value': 42,
-            'context': {'foo': 'FOOBAR', 'bar': 42},
-        }
-    ]
-
-
-def test_pydantic_value_error_invalid_dict():
-    def f(input_value, **kwargs):
-        raise PydanticValueError('my_error', 'this is a custom error {foo}', {(): 'foobar'})
-
-    v = SchemaValidator({'type': 'function', 'mode': 'plain', 'function': f})
-
-    with pytest.raises(ValidationError) as exc_info:
-        v.validate_python(42)
-
-    assert str(exc_info.value) == (
-        '1 validation error for function-plain\n'
-        "  (error rendering message: TypeError: 'tuple' object cannot be converted to 'PyString') "
-        '[kind=my_error, input_value=42, input_type=int]'
-    )
-    with pytest.raises(TypeError, match="'tuple' object cannot be converted to 'PyString'"):
-        exc_info.value.errors()
-
-
-def test_pydantic_value_error_invalid_type():
-    def f(input_value, **kwargs):
-        raise PydanticValueError('my_error', 'this is a custom error {foo}', [('foo', 123)])
-
-    v = SchemaValidator({'type': 'function', 'mode': 'plain', 'function': f})
-
-    with pytest.raises(TypeError, match="argument 'context': 'list' object cannot be converted to 'PyDict'"):
-        v.validate_python(42)
-
-
-def test_validator_instance_plain():
-    class CustomValidator:
-        def __init__(self):
-            self.foo = 42
-            self.bar = 'before'
-
-        def validate(self, input_value, **kwargs):
-            return f'{input_value} {self.foo} {self.bar}'
-
-    c = CustomValidator()
-    v = SchemaValidator({'type': 'function', 'mode': 'plain', 'validator_instance': c, 'function': c.validate})
-    c.foo += 1
-
-    assert v.validate_python('input value') == 'input value 43 before'
-    c.bar = 'changed'
-    assert v.validate_python('input value') == 'input value 43 changed'
-
-
-def test_validator_instance_after():
-    class CustomValidator:
-        def __init__(self):
-            self.foo = 42
-
-        def validate(self, input_value, **kwargs):
-            assert isinstance(input_value, str)
-            return f'{input_value} {self.foo}'
-
-    c = CustomValidator()
-    v = SchemaValidator(
-        {
-            'type': 'function',
-            'mode': 'after',
-            'validator_instance': c,
-            'function': c.validate,
-            'schema': {'type': 'str'},
-        }
-    )
-    c.foo += 1
-
-    assert v.validate_python('input value') == 'input value 43'
-    assert v.validate_python(b'is bytes') == 'is bytes 43'
